'
' 4-Wire Resistive Touchscreen Macros
' Touchscreen mounted on a 128x64 KS0108 Graphical LCD.
'
' The 4-Wire Touchscreen is connected directly to AN0, AN1, AN2, and AN3 of the PICmicro
' With 10K Ohm pull-down resistors on each ADC input.
'
' Touchscreen Cable Configuration:-
'
'    X1 ---------\
'                 \
'    Y1 ---------\ \------O To RA0 (AN0)
'  From           \-------O To RA1 (AN1)
'  Touchscreen    /-------O To RA2 (AN2)
'    X2 ---------/ /------O To RA3 (AN3)
'                 /
'    Y2 ---------/
'
' For use with 18F devices only,
' and the Crownhill Associates Proton+ BASIC Compiler version 3.3.0.1 onwards
'
' Define some global variables used by the Touchscreen subroutines
'
    Dim Xminimum As Word System
    Dim Yminimum As Word System
    Dim Ymaximum As Word System
    Dim Xrange As Word System
    Dim Yrange As Word System
    Dim Xlocation As Byte System
    Dim Ylocation As Byte System
	Dim Xprevious As Byte System
    Dim Yprevious As Byte System
    Dim XposRawRes As Word System
    Dim YposRawRes As Word System

    Dim HotSpotCoords As Dword System
	Dim CoordsXStart As HotSpotCoords.Byte0
	Dim CoordsYStart As HotSpotCoords.Byte1
	Dim CoordsXEnd As HotSpotCoords.Byte2
	Dim CoordsYEnd As HotSpotCoords.Byte3
	Dim BlinkFlag As Bit

	Dim LFSTempWord As PRODL.Word				' Use SFR PRODL\H as a temporary storage variable
'
'----------------------------------------------------------------------------------------------
' Eeprom Data storage
'
Calibrated_Data 	EData As Byte 0
XMinimum_Data 		EData As Word 0
YMinimum_Data 		EData As Word 0
YMaximum_Data 		EData As Word 0
XRange_Data			EData As Word 0
YRange_Data			EData As Word 0

'---------------------------------------------------------------------------------------------
' Initialise the ADC and Ports before entering the main program
' This section may require changing for different PICmicro devices
' Always check the datasheet for the appropriate device.
'
Initialise:
    All_Digital = True							' Make sure the rest of the analoque pins are digital mode first
	ADCON0 = %11000001							' 1 \ FRC (clock derived from the internal ADC RC oscillator)
  												' 1 /
												' 0 \
												' 0   Analog Channel 0 (AN0) Select
												' 0 /
												' 0 - GO_DONE: A/D Conversion clear
												' 0 - Unimplemented
												' 1 - A/D Converter module is powered up

	ADCON1 = %10000010							' 1 - Right justified (Six Most Significant bits of ADRESH are read as 0)
	 											' 0 - Clear A/D Conversion Clock Select bit
												' 0 \
												' 0 / Unimplemented
												' 0 \
												' 0   ADC Port Configuration Control
												' 1   All PORTA Analogue, PORTE Digital
												' 0 /
	TRISA = 0									' PortA is Output
    LATA = 0									' PortA = Low
    GoTo OverTouchScreenSubs					' Jump over the subroutines
'----------------------------------------------------------------------------------------------
' Calibrate the Touch Screen
' Flashes a cursor Top-Left then Bottom-Right of the screen
' When each cursor appears, the stylus must be placed upon it
'
' Syntax:	CalibrateScreen()

$define CalibrateScreen() _mCalibrateScreen
_mCalibrateScreen Macro
    GoSub _CalibrateScreen
Endm
'
' Input		: None
' Output	: Xminimum, Yminimum, Ymaximum, Xrange, YRange
' Notes		: Stores the above values into eeprom once the calibration has taken place
'
#ifdef _mCalibrateScreen#Req
_CalibrateScreen:
@ GetRawXpos#Req = 1							; Enable the _GetRawXpos subroutine
@ GetRawYpos#Req = 1							; Enable the _GetRawYpos subroutine
	BlinkFlag = 1
	Repeat
        GoSub _GetRawXpos                        ' Wait for stylus release
    Until XposRawRes <= 1
	DelayMS 30                                  ' Debounce delay
	Repeat
        If BlinkFlag = 1 Then					' Flash a cursor in the top-left corner
        	LCDWrite 0,0,[15,15,15,15]
    	Else
        	LCDWrite 0,0,[0,0,0,0]
    	EndIf
		GoSub _GetRawXpos                      	' Wait for the stylus to touch the screen
		DelayMS 300
       	BlinkFlag = ~BlinkFlag
    Until XposRawRes > 0
    GoSub _GetRawYpos
    Yminimum = YposRawRes                     	' Save Y ADC value as minimum
    GoSub _GetRawXpos
    Xminimum = XposRawRes                  		' Save X ADC value as minimum
    LCDWrite 0,0,[0,0,0,0]						' Remove the top-left cursor
	Repeat
        GoSub _GetRawXpos                       ' Wait for stylus release
    Until XposRawRes <= 1
	DelayMS 30                                  ' Debounce delay
    Repeat
       	If BlinkFlag = 1 Then					' Flash a cursor in the bottom-right corner
        	LCDWrite 7,124,[240,240,240,240]
    	Else
        	LCDWrite 7,124,[0,0,0,0]
    	EndIf
	   GoSub _GetRawXpos                        ' Wait for the stylus to touch the screen
	   DelayMS 300
       BlinkFlag = ~BlinkFlag
    Until XposRawRes > 0
    GoSub _GetRawYpos
    Ymaximum = YposRawRes                    	' Save Y ADC value as maximum
    Yrange = Ymaximum - Yminimum             	' Calculate Y ADC range across the LCD
	GoSub _GetRawXpos
    Xrange = XposRawRes - Xminimum           	' Calculate X ADC range across the LCD
	LCDWrite 7,124,[0,0,0,0]					' Remove the bottom-right cursor
	Repeat
        GoSub _GetRawXpos                  		' Wait for stylus release
    Until XposRawRes = 0
'
' Write the calibration values to the on-board eeprom
'
	EWrite Calibrated_Data,[1]
	EWrite XMinimum_Data,[Xminimum]
	EWrite YMinimum_Data,[Yminimum]
	EWrite YMaximum_Data,[Ymaximum]
	EWrite XRange_Data,[Xrange]
	EWrite YRange_Data,[Yrange]
    Return
#endif
'----------------------------------------------------------------------------------------------
' Test for the stylus touching the LCD and calculate the stylus position
'
' Syntax:
' Var = LookForStylus
'
' Will return a value of 1 if the stylus is touching the screen, else returns 0
'
$define LookForStylus() _mLookForStylus
_mLookForStylus Macro
	GoSub _LookForStylus
	#if(_mLookForStylus_Return == 1)						' Is there a return value
    	#if(Return_Type == Bit)
        	Wreg_Bit Return_Var,Return_Bit
        #endif
		#if(Return_Type == Byte)
        	#if(Return_Var != WREG)
            	Wreg_Byte Return_Var
            #endif
        #endif
        #if(Return_Type == Word)
        	Wreg_Word Return_Var
        #endif
        #if(Return_Type == Dword)
        	Wreg_Dword Return_Var
        #endif
    #endif
	Endm
'
' Input		: None
' Output	: WREG is 1 if the stylus is touching the LCD, else 0
'			: Byte "Xlocation" holds the X position of the stylus
'			: Byte "Ylocation" holds the Y position of the stylus
'
#ifdef _mLookForStylus#Req
_LookForStylus:
@ GetRawXpos#Req = 1									; Enable the _GetRawXpos subroutine
@ GetRawYpos#Req = 1									; Enable the _GetRawYpos subroutine
    GoSub _GetRawXpos									' Get the raw X ADC value
    If XposRawRes > 0 Then                      		' Is the stylus down?
        DelayMS 30                              		' Yes. So debounce delay
        GoSub _GetRawYpos								' Get the raw Y ADC value
        GoSub _GetRawXpos								' Get the raw X ADC value
        If YposRawRes > Yminimum Then               	' \ Is the stylus within the Y range of the screen?
            If YposRawRes < Ymaximum Then				' /
                LFSTempWord =  XposRawRes - Xminimum    ' \
                LFSTempWord = LFSTempWord << 7			'   Calculate the stylus X location (n * 128)
				Xlocation = LFSTempWord / Xrange		' /
                If Xlocation > 127 Then Xlocation = 127	' Keep the stylus within the LCD's X boundary
                LFSTempWord = YposRawRes - Yminimum     ' \
                LFSTempWord = LFSTempWord << 6			'   Calculate the stylus Y location (n * 64)
				Ylocation = LFSTempWord / Yrange		' /
                If Ylocation > 63 Then Ylocation = 63	' Keep the stylus within the LCD's Y boundary
            	WREG = 1								' Indicate Stylus is down
			EndIf
        EndIf
    Else
		WREG = 0
	EndIf
    Return
#endif
'----------------------------------------------------------------------------------------------
' Get the Raw X position from the ADC
'
' Input		: None
' Output	: Word "XposRawRes" holds the modified ADC value
'
#if GetRawXpos#Req
_GetRawXpos:
    TRISA = TRISA & %11110000                   ' Mask and set to output ADC pins
    TRISA = TRISA | %00000010					' RA1(AN1) is input
    LATA = %00000101                            ' Energise the Y plate and read the X plate
    XposRawRes = ADIn 1                         ' Read the voltage drop on Y+
    If XposRawRes > 0 Then                 		' Is the stylus down?
        TRISA.1 = 0                             ' Yes. So RA1(AN3) is output
        TRISA.3 = 1								' RA3(AN1) is Input
        LFSTempWord = 1023 - ADIn 3            	' Read the voltage drop from the opposite side
        XposRawRes = XposRawRes + LFSTempWord  	' \
        XposRawRes = XposRawRes / 2             ' / Get the average reading
    EndIf
	Return
#endif
'----------------------------------------------------------------------------------------------
' Get the Raw Y position from the ADC
'
' Input		: None
' Output	: Word "YposRawRes" holds the modified ADC value
'
#ifdef GetRawYpos#Req
_GetRawYpos:
    TRISA = TRISA & %11110000                   ' Mask and set to output ADC pins
    TRISA = TRISA | %00000100					' RA2(AN2) is input
    LATA = %00001010                            ' Energise the X plate and read the Y plate
    YposRawRes = ADIn 2                         ' Read the voltage drop on X-
	If YposRawRes > 0 Then                      ' Is the stylus down?
       TRISA.2 = 0                              ' Yes. So RA2(AN2) is output
        TRISA.0 = 1								' RA0(AN0) is input
        LFSTempWord = 1023 - ADIn 0            	' Read the voltage drop from the opposite side
        YposRawRes = YposRawRes + LFSTempWord  	' \
        YposRawRes = YposRawRes / 2             ' / Get the average reading
    EndIf
	Return
#endif
'----------------------------------------------------------------------------------------------
' Check of the Stylus is within a certain area of the LCD
' Create a macro to pass the coordinates to the subroutine and return a value
'
' Syntax:
' Var = Hotspot Top-Left X coord, Top-Left Y coord, Bottom-Right X coord, Bottom-Right Y coord
'
' pXposStart, pYposStart
'           X------------------------
'           |                       |
'           |                       |
'           |                       |
'           |                       |
'           ------------------------X
'                           pXposEnd, pYposEnd
'
' Will return 1 if the stylus is located within the area designated
'
$define HotSpot(pXS, pYS, pXE, pYE) _mHotSpot pXS, pYS, pXE, pYE
_mHotSpot Macro pXposStart, pYposStart, pXposEnd, pYposEnd
	#if((Prm_1 == Num8) || (Prm_1 == Num16) || (Prm_1 == Num32))
		Num_Byte pXposStart , CoordsXStart
	#endif
	#if((Prm_1 == Byte) || (Prm_1 == Word) || (Prm_1 == Dword))
		Byte_Byte pXposStart , CoordsXStart
	#endif

	#if((Prm_2 == Num8) || (Prm_2 == Num16) || (Prm_2 == Num32))
		Num_Byte pYposStart , CoordsYStart
	#endif
	#if((Prm_2 == Byte) || (Prm_2 == Word) || (Prm_2 == Dword))
		Byte_Byte pYposStart , CoordsYStart
	#endif

	#if((Prm_3 == Num8) || (Prm_3 == Num16) || (Prm_3 == Num32))
		Num_Byte pXposEnd , CoordsXEnd
	#endif
	#if((Prm_3 == Byte) || (Prm_3 == Word) || (Prm_3 == Dword))
		Byte_Byte pXposEnd , CoordsXEnd
	#endif

	#if((Prm_4 == Num8) || (Prm_4 == Num16) || (Prm_4 == Num32))
		Num_Byte pYposEnd , CoordsYEnd
	#endif
	#if((Prm_4 == Byte) || (Prm_4 == Word) || (Prm_4 == Dword))
		Byte_Byte pYposEnd , CoordsYEnd
	#endif
	GoSub _HotSpot
	#if(_mHotSpot_Return == 1)					; Is there a return value
    	#if(Return_Type == Bit)
        	Wreg_Bit Return_Var,Return_Bit
        #endif
		#if(Return_Type == Byte)
        	#if(Return_Var != WREG)
            	Wreg_Byte Return_Var
            #endif
        #endif
        #if(Return_Type == Word)
        	Wreg_Word Return_Var
        #endif
        #if(Return_Type == Dword)
        	Wreg_Dword Return_Var
        #endif
    #endif
	Endm
'
' Input		: Byte "CoordsXStart" is the Top-Left X coordinate of the area to check
'			: Byte "CoordsYStart" is the Top-Left Y coordinate of the area to check
'			: Byte "CoordsXEnd" is the Bottom-Right X coordinate of the area to check
'			: Byte "CoordsYEnd" is the Bottom-Right Y coordinate of the area to check
' Output 	: WREG = 1 if Stylus is within the area, else 0
'
#ifdef _mHotSpot#Req
_HotSpot:
	If Xlocation >= CoordsXStart Then
		If Ylocation >= CoordsYStart Then
			If Xlocation <= CoordsXEnd Then
				If Ylocation <= CoordsYEnd Then
					WREG = 1
					Return
				EndIf
			EndIf
		EndIf
	EndIf
	WREG = 0
	Return
#endif
'----------------------------------------------------------------------------------------------
OverTouchScreenSubs:

